/* Metronomos - Copyright (C) 2005 bakunin - Andrea Marchesini 
 *                                   <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include "metronomos.h"

static float config_freq (char *);

/* Create a minimal struct data for simple playing... */
struct metronomos *
config_time (char *timer_str)
{
  struct metronomos *conf;
  struct events *event;
  int timer = atoi (timer_str);

  if (!timer)
    timer = DEFAULT_SPEED;

  if (!(conf = (struct metronomos *) malloc (sizeof (struct metronomos))))
    fatal ("Error: memory.");
  memset (conf, 0, sizeof (struct metronomos));

  if (!(conf->description = strdup ("Standard Metronomos")))
    fatal ("Error: memory.");

  conf->volume = 1.0;

  conf->click_type = CLICK_DEFAULT;

  if (!(event = (struct events *) malloc (sizeof (struct events))))
    fatal ("Error: memory.");
  memset (event, 0, sizeof (struct events));

  if (!(event->description = strdup ("Standard Metronomos")))
    fatal ("Error: memory.");

  event->type = TYPE_NORMAL;

  event->speed = timer;

  conf->event = event;

  return conf;
}

/* Read the config file with libconfuse */
struct metronomos *
config_read (char *file)
{
  struct metronomos *conf;
  struct events *event;
  cfg_t *cfg = NULL, *s_cfg = NULL;
  struct events *last = NULL;
  char *c;
  int i, j;

  cfg_opt_t event_opts[] = {
    CFG_STR ("description", NULL, CFGF_NONE),
    CFG_STR ("type", NULL, CFGF_NONE),

    CFG_STR ("speed", NULL, CFGF_NONE),
    CFG_STR ("click", NULL, CFGF_NONE),
    CFG_STR ("on_click", NULL, CFGF_NONE),
    CFG_STR ("mask_click", NULL, CFGF_NONE),
    CFG_STR ("end_on_seconds", NULL, CFGF_NONE),
    CFG_STR ("end_on_times", NULL, CFGF_NONE),

    CFG_END ()
  };

  cfg_opt_t opts[] = {
    CFG_STR ("description", NULL, CFGF_NONE),

    CFG_STR ("volume", NULL, CFGF_NONE),

    CFG_STR ("click_low", NULL, CFGF_NONE),
    CFG_STR ("click_high", NULL, CFGF_NONE),

    CFG_STR ("click_type", NULL, CFGF_NONE),

    CFG_STR ("click_high_sin_freq", NULL, CFGF_NONE),
    CFG_STR ("click_low_sin_freq", NULL, CFGF_NONE),

    CFG_SEC ("event", event_opts, CFGF_MULTI),
    CFG_END ()
  };

  if (!(conf = (struct metronomos *) malloc (sizeof (struct metronomos))))
    fatal ("Error: memory.");
  memset (conf, 0, sizeof (struct metronomos));

  cfg = cfg_init (opts, CFGF_NOCASE);
  switch (cfg_parse (cfg, file))
    {
    case CFG_FILE_ERROR:
      fatal ("Error: file does not found: %s", file);
      break;

    case CFG_PARSE_ERROR:
      fatal ("Error: syntax error: %s", file);
      break;
    }

  c = cfg_getstr (cfg, "description");
  if (!c || !*c)
    conf->description = strdup ("No description");
  else
    conf->description = strdup (c);

  if (!conf->description)
    fatal ("Error: memory.");

  c = cfg_getstr (cfg, "volume");
  if (!c || !*c || !atoi (c))
    conf->volume = 1.0;
  else
    conf->volume = (float) atoi (c) / 10.0;

  if (conf->volume < 0.0)
    conf->volume = 0.0;
  else if (conf->volume > 1.0)
    conf->volume = 1.0;

  c = cfg_getstr (cfg, "click_type");
  if (!c || !*c || !strcmp (c, "default"))
    conf->click_type = CLICK_DEFAULT;
  else if (!strcmp (c, "sin"))
    conf->click_type = CLICK_SIN;
  else if (!strcmp (c, "file"))
    conf->click_type = CLICK_FILE;
  else
    conf->click_type = CLICK_DEFAULT;

  c = cfg_getstr (cfg, "click_high");
  if (!c || !*c)
    conf->click_high = NULL;
  else if (!(conf->click_high = strdup (c)))
    fatal ("Error: memory.");

  c = cfg_getstr (cfg, "click_high_sin_freq");
  if (!(conf->click_high_sin_freq = config_freq (c)))
    conf->click_high_sin_freq = FREQ_HIGH;

  c = cfg_getstr (cfg, "click_low");
  if (!c || !*c)
    conf->click_low = NULL;
  else if (!(conf->click_low = strdup (c)))
    fatal ("Error: memory.");

  c = cfg_getstr (cfg, "click_low_sin_freq");
  if (!(conf->click_low_sin_freq = config_freq (c)))
    conf->click_low_sin_freq = FREQ_LOW;

  j = cfg_size (cfg, "event");
  for (i = 0; i < j; i++)
    {
      s_cfg = cfg_getnsec (cfg, "event", i);

      if (!(event = (struct events *) malloc (sizeof (struct events))))
	fatal ("Error: memory.");
      memset (event, 0, sizeof (struct events));

      c = cfg_getstr (s_cfg, "description");
      if (!c || !*c)
	event->description = strdup ("No description");
      else
	event->description = strdup (c);

      if (!event->description)
	fatal ("Error: memory.");

      c = cfg_getstr (s_cfg, "type");
      if (!c || !*c)
	event->type = TYPE_NORMAL;
      else
	{
	  if (!strcasecmp (c, "silence"))
	    event->type = TYPE_SILENCE;
	  else if (!strcasecmp (c, "skip"))
	    event->type = TYPE_SKIP;
	  else
	    event->type = TYPE_NORMAL;
	}

      c = cfg_getstr (s_cfg, "speed");
      if (!c || !*c || !atoi (c))
	event->speed = DEFAULT_SPEED;
      else
	event->speed = atoi (c);

      c = cfg_getstr (s_cfg, "mask_click");
      if (!c || !*c)
	event->mask_click = NULL;
      else if (!(event->mask_click = strdup (c)))
	fatal ("Error: memory.");

      c = cfg_getstr (s_cfg, "click");
      if (!c || !*c || !atoi (c))
	event->click = 1;
      else
	event->click = atoi (c);

      c = cfg_getstr (s_cfg, "on_click");
      if (!c || !*c || !atoi (c))
	event->on_click = 4;
      else
	event->on_click = atoi (c);

      if (event->mask_click && strlen (event->mask_click) < event->on_click)
	fatal ("Error: mask_click in %s is minor of on_click tag!",
	       event->description);

      c = cfg_getstr (s_cfg, "end_on_seconds");
      if (!c || !*c || !atoi (c))
	event->end_on_seconds = 0;
      else
	event->end_on_seconds = atoi (c);

      c = cfg_getstr (s_cfg, "end_on_times");
      if (!c || !*c || !atoi (c))
	event->end_on_times = 0;
      else
	event->end_on_times = atoi (c);

      if (event->speed < 0)
	event->speed = DEFAULT_SPEED;
      if (event->click < 0)
	event->click = 0;
      if (event->on_click < 0)
	event->on_click = 0;
      if (event->end_on_seconds < 0)
	event->end_on_seconds = 0;
      if (event->end_on_times < 0)
	event->end_on_times = 0;

      if (event->click > event->on_click)
	event->click = 0;

      if (event->end_on_times && event->end_on_seconds)
	{
	  fprintf (stdout,
		   "Event %s has 2 end elements selected! I use end_on_times\n",
		   event->description);
	  event->end_on_seconds = 0;
	}

      if (!last)
	conf->event = event;
      else
	last->next = event;

      last = event;
    }

  cfg_free (cfg);

  return conf;
}

/* Print on stdout the config of your file */
void
config_write (struct metronomos *metro)
{
  struct events *event;

  event = metro->event;

  fprintf (stdout, "Metronomos: %s\n\n", metro->description);

  while (event)
    {
      fprintf (stdout, "Event: %s\n", event->description);
      fprintf (stdout, "\tType: %s\n",
	       event->type == TYPE_SILENCE ? "silence" : "normal");
      fprintf (stdout, "\tSpeed: %3.d - Click: %3.d - OnClick: %3.d\n",
	       event->speed, event->click, event->on_click);
      fprintf (stdout, "\tEnd after Seconds: %d - End after Times: %d\n",
	       event->end_on_seconds, event->end_on_times);

      fprintf (stdout, "\n");
      event = event->next;
    }
}

static float
config_freq (char *str)
{
  if (!str || !*str || !atol (str))
    return 0;

  return atol (str);
}

/* EOF */
