/* Metronomos - Copyright (C) 2005 bakunin - Andrea Marchesini 
 *                                   <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include "metronomos.h"

static void usage (void);

/* Fatal function... */
void
fatal (char *text, ...)
{
  va_list va;

  va_start (va, text);

  vfprintf (stderr, text, va);
  fprintf (stderr, "\n");

  va_end (va);

  exit (1);
}


int
main (int argc, char **argv)
{
  char *config_file = NULL;
  char *use_time = NULL;
  char *wav_file = NULL;
  int verbose = 0;
  struct metronomos *metro;
  int i;

  fprintf (stdout, "%s %s - %s %s\n\n", PACKAGE, VERSION, AUTHOR, COPYRIGHT);

  for (i = 1; i < argc; i++)
    {
      if (!strcmp (argv[i], "-f") || !strcmp (argv[i], "--file"))
	{
	  if (!(config_file = argv[i + 1]))
	    {
	      usage ();
	      return 1;
	    }

	  i++;
	}

      else if (!strcmp (argv[i], "-t") || !strcmp (argv[i], "--time"))
	{
	  if (!(use_time = argv[i + 1]))
	    {
	      usage ();
	      return 1;
	    }
	  i++;
	}

      else if (!strcmp (argv[i], "-w") || !strcmp (argv[i], "--wav"))
	{
	  if (!(wav_file = argv[i + 1]))
	    {
	      usage ();
	      return 1;
	    }
	  i++;
	}

      else if (!strcmp (argv[i], "-v") || !strcmp (argv[i], "--verbose"))
	{
	  verbose = 1;
	}

      else if (!strcmp (argv[i], "-h") || !strcmp (argv[i], "--help"))
	{
	  usage ();
	  return 0;
	}

      else
	{
	  usage ();
	  return 1;
	}
    }

  /* Or use a file or play a time */
  if (config_file && use_time)
    fatal ("Or file or time! Run %s -h", argv[0]);

  if (use_time)
    /* Set a default config struct data */
    metro = config_time (use_time);

  else
    {
      if (!config_file)
	config_file = DEFAULT_CONFIG_FILE;

      /* Read the config file... */
      metro = config_read (config_file);
    }

  /* Set the cli flags: */
  metro->wav_file = wav_file;
  metro->verbose = verbose;

  /* Exit if no events configurated */
  if (!metro->event)
    fatal ("No event configurated!");

  /* Print config: */
  config_write (metro);

  /* Read the wav file or create the sin() tracks */
  play_read_file (metro);

  /* Set volumes */
  if (metro->volume != 1.0)
    play_set_volume (metro);

  /* Stop signal */
  signal_off (metro);

  /* Open Audio */
  audio_open (metro);

  /* Work: */
  events_start (metro);

  /* Close audio */
  audio_close (metro);

  return 0;
}

static void
usage (void)
{
  fprintf (stdout, "\t-f or --file <file>         Set your config file\n");
  fprintf (stdout,
	   "\t-t or --time <speed>        Don't use the file but play your time\n");
  fprintf (stdout,
	   "\t-w or --wav <file>          Save the output on the <file>.wav\n");
  fprintf (stdout,
	   "\t-v or --verbose             Active a versbose output\n");

  fprintf (stdout, "\n");
}

/* EOF */
