/* Metronome - Copyright (C) 2005 bakunin - Andrea Marchesini 
 *                                    <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _METRONOME_H_
#define _METRONOME_H_

#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <dirent.h>
#include <errno.h>
#include <math.h>
#include <ao/ao.h>
#include <sndfile.h>
#include <limits.h>

#include "../confuse/confuse.h"

#define DEFAULT_CONFIG_FILE	"/etc/metronomos.cfg"
#define AUTHOR			"Andrea Marchesini <bakunin@autistici.org>"
#define COPYRIGHT		"copyright GPL 2005"

#define ONE_SEC 		1000000
#define FREQ_HIGH		880.00
#define FREQ_LOW		770.00
#define DEFAULT_SPEED		60

#define TYPE_NORMAL	1
#define TYPE_SILENCE	2
#define TYPE_SKIP	3

#define CLICK_DEFAULT	0
#define CLICK_SIN	1
#define CLICK_FILE	2

/* Event struct: */
struct events {
  char *description; /* Any event has a description */

  int type; /* This event must play or no ?
	     * TYPE_NORMAL - yes
	     * TYPE_SILENCE - no
	     */

  int speed; /* Speed: 120 bpm */

  int click; /* where is the high click?
	      * 0 -> No high click,
	      * 1 ... on_click -> play the high click here
	      */
  char *mask_click; /* Where is a upcase letter, metronomos
		     * plays a high click, else a low click.
		     */

  int on_click; /* Time division (3/4 -> 3) */

  int end_on_seconds; /* Stop this event after 10 seconds... */
  int end_on_times;   /* ... or after 4 * on_click */
  

  struct events *next; /* This struct creates a list */

} events;


/* Main struct of data */
struct metronomos {

  struct events *event; /* List of events */

  char *description; /* Description of this track */
  int verbose; /* verbose ? 0 (off) or 1 (on) */

  float volume; /* Volume between 0.0 and 1.0 */

  int rate; /* Rate: 44100... */
  int channels; /* Channels: 1 or 2 */

  int click_type; /* Type can be:
      		   * CLICK_DEFAULT - use default.h sound
		   * CLICK_SIN - use sin sound
		   * CLICK_FILE - use a specify wav file
		   */

  char *click_high; /* If you set CLICK_FILE, this is the name of wav file */
  void *click_high_data; /* metronomos reads all wav file and writes it 
			  * in memory (libsndfile) */
  int click_high_size; /* Size... */
  float click_high_durate; /* Durate... */
  float click_high_sin_freq; /* If you set CLICK_SIN, this is the frequence of
			      * high click */

  char *click_low; 	/* This... */
  void *click_low_data; 	/* ...is... */
  int click_low_size; 			/* ...high... */
  float click_low_durate; 			/* ...click... */
  float click_low_sin_freq;

  char *wav_file; /* If you want as output a wav file, this is the name... */
} metronomos;

/* main.c */
void			fatal			(char *, ...);

/* config.c */
struct metronomos *	config_time		(char *);
struct metronomos *	config_read		(char *);
void			config_write		(struct metronomos *);

/* events.c */
void			events_start		(struct metronomos *);

/* play.c */
void			play_high		(struct metronomos *,
						 int, int, int, int);
void			play_low		(struct metronomos *,
						 int, int, int, int);
void			play_write		(void *, size_t);
void			play_read_file		(struct metronomos *);
void			play_set_volume		(struct metronomos *);
void			play_silence		(struct metronomos *,
						 int, int, int);

/* audio.c */
void			audio_open		(struct metronomos *);
void			audio_close		(struct metronomos *);
void			audio_write		(struct metronomos *,
						 void *, size_t);

/* signal.c */
void			signal_off		(struct metronomos *);

#endif

/* EOF */
